﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;

namespace Atac.TemplateEngine.Tests {
  public class Node : INode {
    private readonly NodeKind kind;
    private readonly Node[] children;

    public Node(NodeKind kind) {
      this.kind = kind;
      children = new Node[2];
    }

    public Node(NodeKind kind, object value) : this(kind) {
      Value = value;
    }

    public Node Left {
      get { return children[0]; }
      set { children[0] = value; }
    }
    public Node Right {
      get { return children[1]; }
      set { children[1] = value; }
    }
    public object Value { get; set; }

    public string Kind {
      get { return kind.ToString(); }
    }

    public Node Next { get; set; }

    INode INode.Next {
      get { return Next; }
    }
    IList<INode> INode.Children {
      get { return children; }
    }

    public override string ToString() {
      return Value != null
          ? string.Format("{0}[{1}]", kind, Value)
          : string.Format("{0}", kind);
    }
  }
}