﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;
using Atac.TemplateEngine.Tests.MemberNodes;
using NUnit.Framework;

namespace Atac.TemplateEngine.Tests {
  internal class MemberNodeTests : TestCase<object> {
    [Test]
    public void TextTemplate() {
      Init(@"
VarDecl =
\""\u0056arDecl\"" \\n
.
");
      object ast = new VarDecl();
      GenerateAndVerify(ast, "\"VarDecl\" \\n");
    }

    [Test]
    public void NewLineInText() {
      Init(@"
VarDecl =
  Line1\n  Line2
.
");
      object ast = new VarDecl();
      GenerateAndVerify(ast, "  Line1\r\n  Line2");
    }


    [Test]
    public void NodeCollection() {
      Init(@"
VarDecl(Type, VarId) =
<Type> <VarId sep"", "">;
.
Integer =
int
.
VarId =
<Name>
.
");
      object ast = new VarDecl {
          Type = new Integer(),
          VarId = new List<VarId> {
              new VarId("x"),
              new VarId("y")
          }
      };
      GenerateAndVerify(ast, "int x, y;");
    }

    [Test]
    public void Placeholder() {
      Init(@"
VarDecl(Type, VarId(InitVal)) =
<Type> <VarId sep"", "">< "" = "" InitVal:IntVal>;
.
VarId=
<Name>
.
Integer=
int
.
IntVal(Value) =
<Value>
.
",
//new FileStream("Resources\\TemplateHandlers.cs", FileMode.Open));
          new TemplateHandlers());
      VarDecl varDecl = new VarDecl();
      varDecl.Type = new Integer();
      varDecl.VarId = new List<VarId> {
          new VarId("public") { InitVal = new IntVal(3) },
      };

      GenerateAndVerify(varDecl, @"
int @public = 3;".TrimStart());
    }

    [Test]
    public void RecursiveTemplate() {
      Init(@"
VarDecl =
<this:VarDecl>
.
"
          );
      GenerateAndVerify(new VarDecl(), string.Empty);
    }

    [Test]
    public void RecursiveTemplateHandler() {
      Init(@"
VarDecl =
<this:VarDeclRecursive>
.
"
       , new TemplateHandlers());
      GenerateAndVerify(new VarDecl(), string.Empty);
    }


    [Test]
    public void NodeAlternative() {
      Init(@"
MethodDecl(MethodId, Statement) = // Comment
<MethodId><Statement>
.
MethodId(ParDecls, ReturnType) =
<ReturnType | ""void ""><Name>(<ParDecls sep"", "">) 
.
ParDecl(Type, Name) =
<Type> <Name>
.
Integer =
int
.
Block(Statements)=
{
<Statements> 
}
.
"
          , new TemplateHandlers());
          //, new FileStream("Resources\\TemplateHandlers.cs", FileMode.Open));
      MethodDecl methodDecl = new MethodDecl();
      methodDecl.MethodId = new MethodId {
          Name = "Foo",
          ParDecls = new List<ParDecl> {
              new ParDecl { Type = new Integer(), Name = "p1" },
              new ParDecl { Type = new Integer(), Name = "p2" },
          }
      };
      methodDecl.Statement = new Block();

      //Node ast = new Node(NodeKind.VarDecl) {
      //  Left = new Node(NodeKind.Integer),
      //  Right = new Node(NodeKind.VarId) {
      //    Left = new Node(NodeKind.IntVal)
      //  }
      //};
      GenerateAndVerify(methodDecl, @"
void Foo(int p1, int p2) {
 
}".TrimStart());
    }
  }
}