﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;

namespace Atac.TemplateEngine.Tests {
  class TestLogger : ConsoleLogger {

    public List<Log> Errors = new List<Log>();
    public List<Log> Warnings = new List<Log>();
    public List<Log> Infos = new List<Log>();

    public override void LogError(string msg) {
      base.LogError(msg);
      Errors.Add(new Log(msg));
    }

    public override void LogError(string msg, int line, int col) {
      base.LogError(msg, line, col);
      Errors.Add(new Log(msg, line, col));
    }

    public override void LogWarning(string msg) {
      base.LogWarning(msg);
      Warnings.Add(new Log(msg));
    }

    public override void LogWarning(string msg, int line, int col) {
      base.LogWarning(msg, line, col);
      Warnings.Add(new Log(msg, line, col));
    }

    public override void LogInfo(string msg) {
      base.LogInfo(msg);
      Infos.Add(new Log(msg));
    }

    public override void LogInfo(string msg, int line, int col) {
      base.LogInfo(msg, line, col);
      Infos.Add(new Log(msg, line, col));
    }
  }

  public class Log {
    public string Message { get; private set; }
    public int Line { get; private set; }
    public int Col { get; private set; }

    public Log(string message) {
      Message = message;
    }

    public Log(string message, int line, int col) : this(message) {
      Line = line;
      Col = col;
    }
  }
}
