﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.IO;
using NUnit.Framework;

namespace Atac.TemplateEngine.Tests {

  [TestFixture]
  public class UpdateTemplateHandlersTests {

    [Test]
    public void UpdateTemplateHandlersFromObject() {
      TemplateHandlers1 handlerObject = new TemplateHandlers1();
      TemplateEngine<Node> te = new TemplateEngine<Node>("Resources\\Empty.tpl", handlerObject);
      te.UpdateTemplateHandlers();
      Assert.AreEqual(2, te.TemplateHandlers.Count);
      Assert.Contains(new TemplateHandler<Node>(handlerObject.CorrectHandler1), te.TemplateHandlers.Values);
      Assert.Contains(new TemplateHandler<Node>(handlerObject.CorrectHandler2), te.TemplateHandlers.Values);
    }

    [Test]
    public void UpdateTemplateHandlersFromType() {
      Type handlerType1 = typeof(TemplateHandlers1);
      Type handlerType2 = typeof(TemplateHandlers2);
      TemplateEngine<Node> te = new TemplateEngine<Node>("Resources\\Empty.tpl", handlerType1, handlerType2);
      te.UpdateTemplateHandlers();
      Assert.AreEqual(3, te.TemplateHandlers.Count);
      Assert.Contains("CorrectHandler1", te.TemplateHandlers.Keys);
      Assert.Contains("CorrectHandler2", te.TemplateHandlers.Keys);
      Assert.Contains("CorrectHandler3", te.TemplateHandlers.Keys);
    }

    [Test]
    public void UpdateTemplateHandlersFromStream() {
      TemplateEngine<Node> te;
      using (FileStream fs = new FileStream("Resources\\TemplateHandlers1.cs", FileMode.Open)) {
        te = new TemplateEngine<Node>("Resources\\Empty.tpl", fs);
        te.UpdateTemplateHandlers();
        Assert.AreEqual(2, te.TemplateHandlers.Count);
        Assert.Contains("CorrectHandler1", te.TemplateHandlers.Keys);
        Assert.Contains("CorrectHandler2", te.TemplateHandlers.Keys);
        fs.Position = 0;
        te.UpdateTemplateHandlers();
        Assert.AreEqual(2, te.TemplateHandlers.Count);
        Assert.Contains("CorrectHandler1", te.TemplateHandlers.Keys);
        Assert.Contains("CorrectHandler2", te.TemplateHandlers.Keys);
      }
    }

    [Test]
    public void UpdateTemplateHandlersFromFile() {
      TemplateEngine<Node> te = new TemplateEngine<Node>("Resources\\Empty.tpl", "Resources\\TemplateHandlers2.cs");
      te.UpdateTemplateHandlers();
      Assert.AreEqual(1, te.TemplateHandlers.Count);
      Assert.Contains("CorrectHandler3", te.TemplateHandlers.Keys);
    }

  }
}
