﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System;

namespace Atac.TemplateEngine {
  public class ConsoleLogger : ILogger {
    private const string MsgFormat = "-- line {0} col {1}: {2}"; // 0=line, 1=column, 2=text
    private const ConsoleColor ErrorColor = ConsoleColor.Red;
    private const ConsoleColor WarningColor = ConsoleColor.Yellow;

    public virtual void LogError(string msg) {
      LogColored(ErrorColor, msg);
    }

    public virtual void LogError(string msg, int line, int col) {
      LogColored(ErrorColor, MsgFormat, line, col, msg);
    }

    public virtual void LogWarning(string msg) {
      LogColored(WarningColor, msg);
    }

    public virtual void LogWarning(string msg, int line, int col) {
      LogColored(WarningColor, MsgFormat, line, col, msg);
    }

    public virtual void LogInfo(string msg) {
      Console.WriteLine(msg);
    }

    public virtual void LogInfo(string msg, int line, int col) {
      Console.WriteLine(MsgFormat, line, col, msg);
    }

    private static void LogColored(ConsoleColor color, string msg, params object[] args) {
      ConsoleColor colorBackup = Console.ForegroundColor;
      Console.ForegroundColor = color;
      Console.WriteLine(msg, args);
      Console.ForegroundColor = colorBackup;
    }
  }
}