using System.Collections.Generic;
using System.Globalization;
using System.Text;



using System;

namespace Atac.TemplateEngine {



public class TemplateParser {
	public const int _EOF = 0;
	public const int _lineBreak = 1;
	public const int _ident = 2;
	public const int _number = 3;
	public const int _blanks = 4;
	public const int _sep = 5;
	public const int _chars = 6;
	public const int _templateEnd = 7;
	public const int maxT = 31;

	const bool T = true;
	const bool x = false;
	const int minErrDist = 2;
	
	public Scanner scanner;
	public Errors  errors;

	public Token t;    // last recognized token
	public Token la;   // lookahead token
	int errDist = minErrDist;

public readonly Dictionary<string, Template> Templates 
      = new Dictionary<string, Template>();
  private static readonly PathDecl EmptyPathDecl = Atac.TemplateEngine.PathDecl.Empty;
  private Template template;
  private readonly HashSet<string> declaredPaths = new HashSet<string>();

  private void SemErr(string msg, params object[] args) {
    errors.SemErr(t.line, t.col, string.Format(msg, args));
  }

  private List<PathItem> GetPath(string name) {
    Stack<PathItem> pathStack = new Stack<PathItem>();
    if (name != string.Empty && name != template.Name) {
      if (!GetPath(name, template.PathDecls, pathStack)) {
        pathStack.Push(new PathItem(name));
      }
    }
    List<PathItem> path = new List<PathItem>(pathStack.Count);
    PathItem[] pathItems = pathStack.ToArray();
    for (int i = pathItems.Length - 1; i >= 0; i--) {
      path.Add(pathItems[i]);
    }
    return path;
  }

  private static bool GetPath(string name, IList<PathDecl> decls, Stack<PathItem> pathStack) {
    if (decls == null) {
      return false;
    }
    for(int i = 0; i < decls.Count; i++) {
      pathStack.Push(new PathItem(decls[i].Name, i));
      if (name == decls[i].Name) {
        return true;
      }
      if (GetPath(name, decls[i].Children, pathStack)) {
        return true;
      }
      pathStack.Pop();
    }
    return false;
  }

 


	public TemplateParser(Scanner scanner) {
		this.scanner = scanner;
		errors = new Errors();
	}

  public TemplateParser(Scanner scanner, ILogger logger)
    : this(scanner) {
    errors.logger = logger;
  }

	void SynErr (int n) {
		if (errDist >= minErrDist) errors.SynErr(la.line, la.col, n);
		errDist = 0;
	}

	public void SemErr (string msg) {
		if (errDist >= minErrDist) errors.SemErr(t.line, t.col, msg);
		errDist = 0;
	}
	
	void Get () {
		for (;;) {
			t = la;
			la = scanner.Scan();
			if (la.kind <= maxT) { ++errDist; break; }

			la = t;
		}
	}
	
	void Expect (int n) {
		if (la.kind==n) Get(); else { SynErr(n); }
	}
	
	bool StartOf (int s) {
		return set[s, la.kind];
	}
	
	void ExpectWeak (int n, int follow) {
		if (la.kind == n) Get();
		else {
			SynErr(n);
			while (!StartOf(follow)) Get();
		}
	}


	bool WeakSeparator(int n, int syFol, int repFol) {
		int kind = la.kind;
		if (kind == n) {Get(); return true;}
		else if (StartOf(repFol)) {return false;}
		else {
			SynErr(n);
			while (!(set[syFol, kind] || set[repFol, kind] || set[0, kind])) {
				Get();
				kind = la.kind;
			}
			return StartOf(syFol);
		}
	}

	
	void TemplateCollection() {
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		Template();
		while (la.kind == 2) {
			Template();
		}
	}

	void Ws() {
		if (la.kind == 4) {
			Get();
		} else if (la.kind == 1) {
			Get();
		} else SynErr(32);
		while (la.kind == 1 || la.kind == 4) {
			if (la.kind == 4) {
				Get();
			} else {
				Get();
			}
		}
	}

	void Template() {
		declaredPaths.Clear();                                
		Expect(2);
		string name = t.val; declaredPaths.Add(name);         
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		template = new Template(name);                        
		if (la.kind == 12) {
			IList<PathDecl> pathDecls;                            
			PathDecls(out pathDecls);
			template.PathDecls = pathDecls;                       
		}
		if (la.kind == 9) {
			LineFeedDecl();
		}
		Expect(8);
		if (la.kind == 4) {
			Get();
		}
		if (la.kind == 1) {
			Get();
			Expression expr;                                      
			Expression(out expr);
			template.Expression = expr;                           
		}
		Expect(7);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		Templates.Add(name, template);                        
	}

	void PathDecls(out IList<PathDecl> pathDecls) {
		pathDecls = new List<PathDecl>();             
		Expect(12);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		PathDecl pathDecl;                            
		if (la.kind == 2) {
			PathDecl(out pathDecl);
			pathDecls.Add(pathDecl);                      
		} else if (la.kind == 10 || la.kind == 13) {
			pathDecls.Add(EmptyPathDecl);                 
		} else SynErr(33);
		while (la.kind == 10) {
			Get();
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
			if (la.kind == 2) {
				PathDecl(out pathDecl);
				pathDecls.Add(pathDecl);                      
			} else if (la.kind == 10 || la.kind == 13) {
				pathDecls.Add(EmptyPathDecl);                 
			} else SynErr(34);
		}
		Expect(13);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
	}

	void LineFeedDecl() {
		Expect(9);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		int precLFs = 0, succLFs = 0;                 
		if (la.kind == 3) {
			Number(out precLFs);
			if (la.kind == 10) {
				Get();
				if (la.kind == 1 || la.kind == 4) {
					Ws();
				}
				Number(out succLFs);
			}
		} else if (la.kind == 10) {
			Get();
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
			Number(out succLFs);
		} else SynErr(35);
		template.PrecedingLineFeeds = precLFs;        
		template.SucceedingLineFeeds = succLFs;       
		Expect(11);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
	}

	void Expression(out Expression expr) {
		expr = new Expression(); Line line = new Line();  
		if (StartOf(1)) {
			LineExpression(line);
		}
		expr.Lines.Add(line);                             
		while (la.kind == 1) {
			line = new Line();                                
			Get();
			if (StartOf(1)) {
				LineExpression(line);
			}
			expr.Lines.Add(line);                             
		}
	}

	void Number(out int value) {
		Expect(3);
		value = errDist > 0 ? int.Parse(t.val) : 0;   
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
	}

	void PathDecl(out PathDecl pathDecl) {
		pathDecl = null;                                  
		Expect(2);
		string name = t.val;                              
		if (declaredPaths.Contains(name)) {               
		SemErr("Path '{0}' already declared", name);    
		}                                                 
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		declaredPaths.Add(name);                          
		if (la.kind == 12) {
			IList<PathDecl> children;                         
			PathDecls(out children);
			pathDecl = new PathDecl(name, children);          
		} else if (la.kind == 10 || la.kind == 13) {
			pathDecl = new PathDecl(name);                    
		} else SynErr(36);
	}

	void LineExpression(Line line) {
		if (la.kind == 4 || la.kind == 14) {
			if (la.kind == 4) {
				Get();
				line.Indentation = t.val;                       
			} else {
				Placeholder(line);
			}
			if (StartOf(2)) {
				Text(line, string.Empty);
			}
		} else if (StartOf(3)) {
			string word;                                    
			Word(out word);
			if (StartOf(2)) {
				Text(line, word);
			} else if (StartOf(4)) {
				line.Elements.Add(new Text(word));              
			} else SynErr(37);
		} else SynErr(38);
		while (la.kind == 14) {
			Placeholder(line);
			if (StartOf(2)) {
				Text(line, string.Empty);
			}
		}
	}

	void Placeholder(Line line) {
		IList<PathItem> path;                             
		Expression preExpr = null;                        
		Expression sepExpr = null;                        
		Expression sufExpr = null;                        
		Expression altExpr = null;                        
		string link = null;                               
		Expect(14);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		if (la.kind == 20) {
			QuoteExpression(out preExpr);
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
		}
		Path(out path);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		if (la.kind == 18) {
			TemplateLink(out link);
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
		}
		if (la.kind == 19) {
			NodeAlternative(out altExpr);
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
		}
		if (la.kind == 5) {
			Get();
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
			QuoteExpression(out sepExpr);
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
		}
		if (la.kind == 20) {
			QuoteExpression(out sufExpr);
			if (la.kind == 1 || la.kind == 4) {
				Ws();
			}
		}
		Expect(15);
		line.Elements.Add(new Placeholder(path,           
		preExpr, sepExpr, sufExpr, altExpr, link));   
	}

	void Text(Line line, string prefix) {
		string word;                                      
		StringBuilder sb = new StringBuilder(prefix);     
		if (la.kind == 4) {
			Get();
			sb.Append(t.val);                                 
		} else if (StartOf(3)) {
			Word(out word);
			sb.Append(word);                                  
		} else SynErr(39);
		while (StartOf(2)) {
			if (la.kind == 4) {
				Get();
				sb.Append(t.val);                                 
			} else {
				Word(out word);
				sb.Append(word);                                  
			}
		}
		line.Elements.Add(new Text(sb.ToString()));       
	}

	void Word(out string word) {
		word = string.Empty;                              
		switch (la.kind) {
		case 2: case 3: case 5: case 6: case 8: case 9: case 10: case 11: case 12: case 13: case 17: case 18: case 19: case 21: case 22: {
			switch (la.kind) {
			case 2: {
				Get();
				break;
			}
			case 3: {
				Get();
				break;
			}
			case 5: {
				Get();
				break;
			}
			case 6: {
				Get();
				break;
			}
			case 8: {
				Get();
				break;
			}
			case 12: {
				Get();
				break;
			}
			case 13: {
				Get();
				break;
			}
			case 10: {
				Get();
				break;
			}
			case 9: {
				Get();
				break;
			}
			case 11: {
				Get();
				break;
			}
			case 18: {
				Get();
				break;
			}
			case 19: {
				Get();
				break;
			}
			case 21: {
				Get();
				break;
			}
			case 22: {
				Get();
				break;
			}
			case 17: {
				Get();
				break;
			}
			}
			word = t.val;                                     
			break;
		}
		case 23: {
			Get();
			word = "\\";                                      
			break;
		}
		case 24: {
			Get();
			word = "<";                                       
			break;
		}
		case 25: {
			Get();
			word = ">";                                       
			break;
		}
		case 26: {
			Get();
			word = "\"";                                      
			break;
		}
		case 27: {
			Get();
			word = ".";                                       
			break;
		}
		case 28: {
			Get();
			word = "/";                                       
			break;
		}
		case 29: {
			Get();
			word = "\t";                                      
			break;
		}
		case 30: {
			Get();
			Expect(3);
			word = char.ConvertFromUtf32(                     
			int.Parse(t.val, NumberStyles.HexNumber));      
			break;
		}
		default: SynErr(40); break;
		}
	}

	void QuoteExpression(out Expression expr) {
		Expect(20);
		Expression(out expr);
		Expect(20);
	}

	void Path(out IList<PathItem> path) {
		path = new List<PathItem>();                      
		if (la.kind == 16) {
			Get();
		} else if (la.kind == 2) {
			Get();
			path = GetPath(t.val);                            
		} else SynErr(41);
		while (la.kind == 17) {
			Get();
			Expect(2);
			path.Add(new PathItem(t.val));                    
		}
	}

	void TemplateLink(out string templateLink) {
		Expect(18);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		Expect(2);
		templateLink = t.val;                             
	}

	void NodeAlternative(out Expression altExpr) {
		Expect(19);
		if (la.kind == 1 || la.kind == 4) {
			Ws();
		}
		QuoteExpression(out altExpr);
	}



	public void Parse() {
		la = new Token();
		la.val = "";		
		Get();
		TemplateCollection();
		Expect(0);

	}
	
	static readonly bool[,] set = {
		{T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x},
		{x,x,T,T, T,T,T,x, T,T,T,T, T,T,T,x, x,T,T,T, x,T,T,T, T,T,T,T, T,T,T,x, x},
		{x,x,T,T, T,T,T,x, T,T,T,T, T,T,x,x, x,T,T,T, x,T,T,T, T,T,T,T, T,T,T,x, x},
		{x,x,T,T, x,T,T,x, T,T,T,T, T,T,x,x, x,T,T,T, x,T,T,T, T,T,T,T, T,T,T,x, x},
		{x,T,x,x, x,x,x,T, x,x,x,x, x,x,T,x, x,x,x,x, T,x,x,x, x,x,x,x, x,x,x,x, x}

	};
} // end Parser


public class Errors {
	public int count = 0;                                    // number of errors detected
  public ILogger logger = new ConsoleLogger();             // error messages go to this logger
  
	public virtual void SynErr (int line, int col, int n) {
		string s;
		switch (n) {
			case 0: s = "EOF expected"; break;
			case 1: s = "lineBreak expected"; break;
			case 2: s = "ident expected"; break;
			case 3: s = "number expected"; break;
			case 4: s = "blanks expected"; break;
			case 5: s = "sep expected"; break;
			case 6: s = "chars expected"; break;
			case 7: s = "templateEnd expected"; break;
			case 8: s = "\"=\" expected"; break;
			case 9: s = "\"[\" expected"; break;
			case 10: s = "\",\" expected"; break;
			case 11: s = "\"]\" expected"; break;
			case 12: s = "\"(\" expected"; break;
			case 13: s = "\")\" expected"; break;
			case 14: s = "\"<\" expected"; break;
			case 15: s = "\">\" expected"; break;
			case 16: s = "\"this\" expected"; break;
			case 17: s = "\".\" expected"; break;
			case 18: s = "\":\" expected"; break;
			case 19: s = "\"|\" expected"; break;
			case 20: s = "\"\"\" expected"; break;
			case 21: s = "\"/\" expected"; break;
			case 22: s = "\"*\" expected"; break;
			case 23: s = "\"\\\\\\\\\" expected"; break;
			case 24: s = "\"\\\\<\" expected"; break;
			case 25: s = "\"\\\\>\" expected"; break;
			case 26: s = "\"\\\\\\\"\" expected"; break;
			case 27: s = "\"\\\\.\" expected"; break;
			case 28: s = "\"\\\\/\" expected"; break;
			case 29: s = "\"\\\\t\" expected"; break;
			case 30: s = "\"\\\\u\" expected"; break;
			case 31: s = "??? expected"; break;
			case 32: s = "invalid Ws"; break;
			case 33: s = "invalid PathDecls"; break;
			case 34: s = "invalid PathDecls"; break;
			case 35: s = "invalid LineFeedDecl"; break;
			case 36: s = "invalid PathDecl"; break;
			case 37: s = "invalid LineExpression"; break;
			case 38: s = "invalid LineExpression"; break;
			case 39: s = "invalid Text"; break;
			case 40: s = "invalid Word"; break;
			case 41: s = "invalid Path"; break;

			default: s = "error " + n; break;
		}
    if (logger != null) {
		  logger.LogError(s, line, col);
    }
		count++;
	}

	public virtual void SemErr (int line, int col, string s) {
    if (logger != null) {
		  logger.LogError(s, line, col);
    }
		count++;
	}
	
	public virtual void SemErr (string s) {
    if (logger != null) {
		  logger.LogError(s);
    }
		count++;
	}
	
	public virtual void Warning (int line, int col, string s) {
    if (logger != null) {
		  logger.LogWarning(s, line, col);
    }
	}
	
	public virtual void Warning(string s) {
    if (logger != null) {
		  logger.LogWarning(s);
    }
	}
} // Errors


public class FatalError: Exception {
	public FatalError(string m): base(m) {}
}
}