﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace Atac.TemplateEngine {
  public class Expression {
    private readonly List<Line> lines = new List<Line>();
    public Expression() {}

    public Expression(IEnumerable<Line> lines) {
      if (lines != null) {
        this.lines.AddRange(lines);
      }
    }

    public List<Line> Lines {
      get { return lines; }
    }

    public bool Put<TNode>(CodeGenerator<TNode> codeGen) where TNode : class {
      if (lines.Count == 0) {
        return false;
      }
      bool result = false;
      ExpressionWriter<TNode> expressionWriter = new ExpressionWriter<TNode>(codeGen);
      foreach (Line line in Lines) {
        result |= expressionWriter.PutLine(line);
      }
      return result;
    }

    public override string ToString() {
      return ToString(Environment.NewLine);
    }

    public string ToString(string newLine) {
      newLine = newLine ?? Environment.NewLine;
      if (lines.Count == 0) {
        return string.Empty;
      }
      StringBuilder sb = new StringBuilder(lines[0].ToString());
      for (int i  = 1; i < lines.Count; i++) {
        sb.Append(lines[i]);
        sb.Append(newLine);
      }
      return sb.ToString();
    }

  }
}