// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;
using System.Text;

namespace Atac.TemplateEngine {
  public class Line {
    private string indentation;
    private readonly List<LineElement> elements = new List<LineElement>();

    public Line() {}
    public Line(string indentation) : this(indentation, null) {}
    public Line(IEnumerable<LineElement> elements) : this(null, elements) {}

    public Line(string indentation, IEnumerable<LineElement> elements) {
      this.indentation = indentation;
      if (elements != null) {
        this.elements.AddRange(elements);
      }
    }

    public string Indentation {
      get { return indentation; }
      set { indentation = value; }
    }

    public List<LineElement> Elements {
      get { return elements; }
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder(indentation);
      foreach (LineElement element in Elements) {
        sb.Append(element);
      }
      return sb.ToString();
    }
  }
}