﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace Atac.TemplateEngine {
  public class PathDecl {
    public static readonly PathDecl Empty;
    static PathDecl() {
      Empty = new PathDecl();
    }

    public string Name { get; private set; }
    public IList<PathDecl> Children { get; private set; }

    private PathDecl(){}

    public PathDecl(string name)
      : this(name, new PathDecl[0]) {}

    public PathDecl(string name, IList<PathDecl> children){
      if (string.IsNullOrEmpty(name)) {
        throw new ArgumentException("Template name must not be null, empty, or consist only of whitespace", "name");
      }
      if (children == null) {
        throw new ArgumentNullException("children");
      }
      Name = name;
      Children = children;
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append(Name);
      if (Children.Count > 0) {
        sb.Append("(");
        sb.Append(Children[0]);
        for (int i = 1; i < Children.Count; i++) {
          sb.Append(", ");
          sb.Append(Children[i]);
        }
        sb.Append(")");
      }
      return sb.ToString();
    }

  }
}