﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;
using System.Text;

namespace Atac.TemplateEngine {
  public class Placeholder : LineElement {
    private IList<PathItem> path;
    private Expression prefix;
    private Expression separator;
    private Expression suffix;
    private Expression alternative;
    private string link;

    public Placeholder() {}

    public Placeholder(IList<PathItem> path) {
      this.path = path;
    }

    public Placeholder(params string[] path) {
      if (path == null) {
        return;
      }
      this.path = new PathItem[path.Length];
      for (int i = 0; i < path.Length; i++) {
        this.path[i] = new PathItem(path[i]);
      }
    }

    public Placeholder(IList<PathItem> path,
        Expression prefix,
        Expression separator,
        Expression suffix,
        Expression alternative,
        string link) : this(path) {
      this.prefix = prefix;
      this.separator = separator;
      this.suffix = suffix;
      this.alternative = alternative;
      this.link = link;
    }

    public IList<PathItem> Path {
      get { return path; }
      set { path = value; }
    }
    public Expression Prefix {
      get { return prefix; }
      set { prefix = value; }
    }
    public Expression Separator {
      get { return separator; }
      set { separator = value; }
    }
    public Expression Suffix {
      get { return suffix; }
      set { suffix = value; }
    }
    public Expression Alternative {
      get { return alternative; }
      set { alternative = value; }
    }
    public string Link {
      get { return link; }
      set { link = value; }
    }


    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("<");
      if (prefix != null && prefix.Lines.Count > 0) {
        sb.Append("\"");
        sb.Append(prefix.ToString("\\n"));
        sb.Append("\" ");
      }
      if (Path != null && Path.Count > 0) {
        sb.Append(Path[0]);
        for (int i = 1; i < Path.Count; i++) {
          sb.Append(".");
          sb.Append(Path[i]);
        }
      }
      if (alternative != null && alternative.Lines.Count > 0) {
        sb.Append(" | \"");
        sb.Append(alternative.ToString("\\n"));
        sb.Append("\"");
      }
      if (separator != null && separator.Lines.Count > 0) {
        sb.Append(" sep\"");
        sb.Append(separator.ToString("\\n"));
        sb.Append("\"");
      }
      if (suffix != null && suffix.Lines.Count > 0) {
        sb.Append(" \"");
        sb.Append(suffix.ToString("\\n"));
        sb.Append("\"");
      }
      sb.Append(">");
      return sb.ToString();
    }

    internal override void Put<TNode>(ExpressionWriter<TNode> writer) {
      writer.PutPlaceholder(this);
    }

  }
}