﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace Atac.TemplateEngine {
  public class BucketTreeHelper : TreeHelper<INode> {
    public override object GetPathValue(INode node, IEnumerable<PathItem> path) {
      List<object> values = new List<object> { node };
      foreach (PathItem pathItem in path) {
        List<object> targets = values;
        values = pathItem.Index >= 0 
          ? GetChildNodes(pathItem, targets, path) 
          : GetFieldValues(pathItem, targets);
      }
      return values.Count > 1
          ? values : values.Count == 1
              ? values[0] : null;
    }

    public override string GetNodeKind(INode node) {
      return node.Kind;
    }

    private List<object> GetChildNodes(PathItem pathItem, List<object> targets, IEnumerable<PathItem> path) {
      List<object> values = new List<object>();
      foreach (object target in targets) {
        if (!(target is INode)) {
          CodeGenerator.LogError(
              "Path '{0}' does not reference a node or a collection of nodes",
              GetPathString(path, pathItem));
          continue;
        }
        INode targetNode = (INode) target;
        int index = pathItem.Index;
        if (targetNode.Children == null
            || targetNode.Children.Count <= index
                || targetNode.Children[index] == null) {
          continue;
        }
        values.AddRange(GetEnumerable(targetNode.Children[index]));
      }
      return values;
    }

    private List<object> GetFieldValues(PathItem pathItem, List<object> targets) {
      List<object> values = new List<object>();
      foreach (object target in targets) {
        object value = GetFieldValue(target, pathItem.Name);
        if (value == null) {
          continue;
        }
        if (value is IEnumerable) {
          foreach (object item in (IEnumerable) value) {
            values.Add(item);
          }
        } else {
          values.Add(value);
        }
      }
      return values;
    }

    private string GetPathString(IEnumerable<PathItem> path, PathItem toPathItem) {
      StringBuilder sb = new StringBuilder();
      IEnumerator<PathItem> enumerator = path.GetEnumerator();
      if (enumerator.MoveNext()) {
        sb.Append(enumerator.Current.Name);
        while (enumerator.MoveNext() && enumerator.Current != toPathItem) {
          sb.Append('.');
          sb.Append(enumerator.Current.Name);
        }
      }
      return sb.ToString();
    }

    private static IEnumerable<object> GetEnumerable(INode node) {
      while (node != null) {
        yield return node;
        node = node.Next;
      }
    }
  }
}