﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections;
using System.Collections.Generic;

namespace Atac.TemplateEngine {
  public class MemberTreeHelper<TNode> : TreeHelper<TNode> where TNode : class {
    public override object GetPathValue(TNode node, IEnumerable<PathItem> path) {
      List<object> values = new List<object> { node };
      foreach (PathItem pathItem in path) {
        List<object> targets = values;
        values = new List<object>();
        foreach (object target in targets) {
          object value = GetFieldValue(target, pathItem.Name);
          if (value == null) {
            continue;
          }
          if (value is IEnumerable) {
            foreach (object item in (IEnumerable) value) {
              values.Add(item);
            }
          } else {
            values.Add(value);
          }
        }
      }
      return values.Count > 1
          ? values : values.Count == 1
              ? values[0] : null;
    }

    public override string GetNodeKind(TNode node) {
      return node.GetType().Name;
    }
  }
}