﻿// Atac Template Engine
// Copyright (C) 2011 Markus Jahn <markus.jahn@gmx.at>
//
// This file is part of Atac Template Engine.
//
// Atac Template Engine is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Atac Template Engine is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Atac Template Engine.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Reflection;

namespace Atac.TemplateEngine {
  public abstract class TreeHelper<TNode> where TNode : class {
    public CodeGenerator<TNode> CodeGenerator { get; private set; }

    protected internal virtual void Init(CodeGenerator<TNode> cg) {
      CodeGenerator = cg;
    }

    public abstract object GetPathValue(TNode node, IEnumerable<PathItem> path); 
    public abstract string GetNodeKind(TNode node);

    protected virtual object GetFieldValue(object target, string fieldName) {
      Type targetType = target.GetType();
      MemberInfo[] memberInfos = targetType.GetMember(
          fieldName, CodeGenerator<TNode>.MEMBER_TYPES, CodeGenerator<TNode>.BINDING_FLAGS);
      if (memberInfos.Length == 0) {
        CodeGenerator.LogError(
            "Data field '{0}' of class '{1}' not found", fieldName, targetType.Name);
        return null;
      }
      object value = memberInfos[0].MemberType == MemberTypes.Property
          ? ((PropertyInfo)memberInfos[0]).GetValue(target, null)
          : ((FieldInfo)memberInfos[0]).GetValue(target);
      return value;
    }
  }
}