﻿//reference:TasteToCs.exe

using Atac.TemplateEngine;

namespace Taste {
  public class TasteToCs {
    public void Proc(Node node, CodeGenerator<Node> cg) {
      if (((Proc) node).program == null) {
        cg.PutTemplate(node, "Prog");
      } else {
        cg.PutTemplate(node);
      }
    }

    public void ProgMem(Node node, CodeGenerator<Node> cg) {
      if (node is Var) {
        cg.PutNode(node, "Field");
      } else {
        cg.PutNode(node);
      }
    }

    public void VarType(Node node, CodeGenerator<Node> cg) {
      Type type = ((Obj)node).type;
      switch (type) {
        case Type.UNDEF:
          cg.LogError("Invalid var type.");
          break;
        case Type.INT:
          cg.PutText("int");
          break;
        case Type.BOOL:
          cg.PutText("bool");
          break;
      }
    }

    public void BinOp(Node node, CodeGenerator<Node> cg) {
      switch (((BinExpr)node).op) {
        case Operator.EQU:
          cg.PutText("==");
          break;
        case Operator.LSS:
          cg.PutText("<");
          break;
        case Operator.GTR:
          cg.PutText(">");
          break;
        case Operator.ADD:
          cg.PutText("+");
          break;
        case Operator.SUB:
          cg.PutText("-");
          break;
        case Operator.MUL:
          cg.PutText("*");
          break;
        case Operator.DIV:
          cg.PutText("/");
          break;
      }
    }

    public void UnaryOp(Node node, CodeGenerator<Node> cg) {
      switch (((UnaryExpr)node).op) {
        case Operator.ADD:
          cg.PutText("+");
          break;
        case Operator.SUB:
          cg.PutText("-");
          break;
        default:
          cg.LogError("Invalid unary operator.");
          break;
      }
    }

    public void Stat(Node node, CodeGenerator<Node> cg) {
      if (cg.PutNode(node) && !IsStructStat(node)) {
        cg.PutText(";");
      }
    }

    private bool IsStructStat(Node node) {
      return node is If
          || node is IfElse
              || node is While
                  || node is Block;
    }
  }
}